# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow_metadata/proto/v0/statistics.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow_metadata.proto.v0 import path_pb2 as tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow_metadata/proto/v0/statistics.proto',
  package='tensorflow.metadata.v0',
  syntax='proto3',
  serialized_options=b'\n\032org.tensorflow.metadata.v0P\001ZEgithub.com/feast-dev/feast/sdk/go/protos/tensorflow_metadata/proto/v0\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n-tensorflow_metadata/proto/v0/statistics.proto\x12\x16tensorflow.metadata.v0\x1a\'tensorflow_metadata/proto/v0/path.proto\"b\n\x1c\x44\x61tasetFeatureStatisticsList\x12\x42\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x30.tensorflow.metadata.v0.DatasetFeatureStatistics\"\xe6\x01\n\x18\x44\x61tasetFeatureStatistics\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0cnum_examples\x18\x02 \x01(\x04\x12\x1d\n\x15weighted_num_examples\x18\x04 \x01(\x01\x12?\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32-.tensorflow.metadata.v0.FeatureNameStatistics\x12\x46\n\x0e\x63ross_features\x18\x05 \x03(\x0b\x32..tensorflow.metadata.v0.CrossFeatureStatistics\"\xb4\x02\n\x16\x43rossFeatureStatistics\x12,\n\x06path_x\x18\x01 \x01(\x0b\x32\x1c.tensorflow.metadata.v0.Path\x12,\n\x06path_y\x18\x02 \x01(\x0b\x32\x1c.tensorflow.metadata.v0.Path\x12\r\n\x05\x63ount\x18\x03 \x01(\x04\x12I\n\x0fnum_cross_stats\x18\x04 \x01(\x0b\x32..tensorflow.metadata.v0.NumericCrossStatisticsH\x00\x12U\n\x17\x63\x61tegorical_cross_stats\x18\x05 \x01(\x0b\x32\x32.tensorflow.metadata.v0.CategoricalCrossStatisticsH\x00\x42\r\n\x0b\x63ross_stats\"A\n\x16NumericCrossStatistics\x12\x13\n\x0b\x63orrelation\x18\x01 \x01(\x02\x12\x12\n\ncovariance\x18\x02 \x01(\x02\"R\n\x1a\x43\x61tegoricalCrossStatistics\x12\x34\n\x04lift\x18\x01 \x01(\x0b\x32&.tensorflow.metadata.v0.LiftStatistics\"\x8b\x01\n\x0eLiftStatistics\x12\x37\n\x0blift_series\x18\x01 \x03(\x0b\x32\".tensorflow.metadata.v0.LiftSeries\x12@\n\x14weighted_lift_series\x18\x02 \x03(\x0b\x32\".tensorflow.metadata.v0.LiftSeries\"\x8d\x04\n\nLiftSeries\x12\x0f\n\x05y_int\x18\x01 \x01(\x05H\x00\x12\x12\n\x08y_string\x18\x02 \x01(\tH\x00\x12=\n\x08y_bucket\x18\x03 \x01(\x0b\x32).tensorflow.metadata.v0.LiftSeries.BucketH\x00\x12\x11\n\x07y_count\x18\x04 \x01(\x04H\x01\x12\x1a\n\x10weighted_y_count\x18\x05 \x01(\x01H\x01\x12\x41\n\x0blift_values\x18\x06 \x03(\x0b\x32,.tensorflow.metadata.v0.LiftSeries.LiftValue\x1a/\n\x06\x42ucket\x12\x11\n\tlow_value\x18\x01 \x01(\x01\x12\x12\n\nhigh_value\x18\x02 \x01(\x01\x1a\xdb\x01\n\tLiftValue\x12\x0f\n\x05x_int\x18\x01 \x01(\x05H\x00\x12\x12\n\x08x_string\x18\x02 \x01(\tH\x00\x12\x0c\n\x04lift\x18\x03 \x01(\x01\x12\x11\n\x07x_count\x18\x04 \x01(\x04H\x01\x12\x1a\n\x10weighted_x_count\x18\x05 \x01(\x01H\x01\x12\x17\n\rx_and_y_count\x18\x06 \x01(\x04H\x02\x12 \n\x16weighted_x_and_y_count\x18\x07 \x01(\x01H\x02\x42\t\n\x07x_valueB\x0f\n\rx_count_valueB\x15\n\x13x_and_y_count_valueB\t\n\x07y_valueB\x0f\n\ry_count_value\"\xae\x04\n\x15\x46\x65\x61tureNameStatistics\x12\x0e\n\x04name\x18\x01 \x01(\tH\x00\x12,\n\x04path\x18\x08 \x01(\x0b\x32\x1c.tensorflow.metadata.v0.PathH\x00\x12@\n\x04type\x18\x02 \x01(\x0e\x32\x32.tensorflow.metadata.v0.FeatureNameStatistics.Type\x12>\n\tnum_stats\x18\x03 \x01(\x0b\x32).tensorflow.metadata.v0.NumericStatisticsH\x01\x12@\n\x0cstring_stats\x18\x04 \x01(\x0b\x32(.tensorflow.metadata.v0.StringStatisticsH\x01\x12>\n\x0b\x62ytes_stats\x18\x05 \x01(\x0b\x32\'.tensorflow.metadata.v0.BytesStatisticsH\x01\x12@\n\x0cstruct_stats\x18\x07 \x01(\x0b\x32(.tensorflow.metadata.v0.StructStatisticsH\x01\x12=\n\x0c\x63ustom_stats\x18\x06 \x03(\x0b\x32\'.tensorflow.metadata.v0.CustomStatistic\"=\n\x04Type\x12\x07\n\x03INT\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\n\n\x06STRING\x10\x02\x12\t\n\x05\x42YTES\x10\x03\x12\n\n\x06STRUCT\x10\x04\x42\n\n\x08\x66ield_idB\x07\n\x05stats\"x\n\x18WeightedCommonStatistics\x12\x17\n\x0fnum_non_missing\x18\x01 \x01(\x01\x12\x13\n\x0bnum_missing\x18\x02 \x01(\x01\x12\x16\n\x0e\x61vg_num_values\x18\x03 \x01(\x01\x12\x16\n\x0etot_num_values\x18\x04 \x01(\x01\"\xbd\x01\n\x0f\x43ustomStatistic\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x03num\x18\x02 \x01(\x01H\x00\x12\r\n\x03str\x18\x03 \x01(\tH\x00\x12\x36\n\thistogram\x18\x04 \x01(\x0b\x32!.tensorflow.metadata.v0.HistogramH\x00\x12?\n\x0erank_histogram\x18\x05 \x01(\x0b\x32%.tensorflow.metadata.v0.RankHistogramH\x00\x42\x05\n\x03val\"\xb9\x02\n\x11NumericStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\x12\x0c\n\x04mean\x18\x02 \x01(\x01\x12\x0f\n\x07std_dev\x18\x03 \x01(\x01\x12\x11\n\tnum_zeros\x18\x04 \x01(\x04\x12\x0b\n\x03min\x18\x05 \x01(\x01\x12\x0e\n\x06median\x18\x06 \x01(\x01\x12\x0b\n\x03max\x18\x07 \x01(\x01\x12\x35\n\nhistograms\x18\x08 \x03(\x0b\x32!.tensorflow.metadata.v0.Histogram\x12Q\n\x16weighted_numeric_stats\x18\t \x01(\x0b\x32\x31.tensorflow.metadata.v0.WeightedNumericStatistics\"\xa2\x03\n\x10StringStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\x12\x0e\n\x06unique\x18\x02 \x01(\x04\x12I\n\ntop_values\x18\x03 \x03(\x0b\x32\x35.tensorflow.metadata.v0.StringStatistics.FreqAndValue\x12\x12\n\navg_length\x18\x04 \x01(\x02\x12=\n\x0erank_histogram\x18\x05 \x01(\x0b\x32%.tensorflow.metadata.v0.RankHistogram\x12O\n\x15weighted_string_stats\x18\x06 \x01(\x0b\x32\x30.tensorflow.metadata.v0.WeightedStringStatistics\x12\x17\n\x0fvocabulary_file\x18\x07 \x01(\t\x1a\x36\n\x0c\x46reqAndValue\x12\r\n\x05value\x18\x02 \x01(\t\x12\x11\n\tfrequency\x18\x03 \x01(\x01J\x04\x08\x01\x10\x02\"\x81\x01\n\x19WeightedNumericStatistics\x12\x0c\n\x04mean\x18\x01 \x01(\x01\x12\x0f\n\x07std_dev\x18\x02 \x01(\x01\x12\x0e\n\x06median\x18\x03 \x01(\x01\x12\x35\n\nhistograms\x18\x04 \x03(\x0b\x32!.tensorflow.metadata.v0.Histogram\"\xa4\x01\n\x18WeightedStringStatistics\x12I\n\ntop_values\x18\x01 \x03(\x0b\x32\x35.tensorflow.metadata.v0.StringStatistics.FreqAndValue\x12=\n\x0erank_histogram\x18\x02 \x01(\x0b\x32%.tensorflow.metadata.v0.RankHistogram\"\xa6\x01\n\x0f\x42ytesStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\x12\x0e\n\x06unique\x18\x02 \x01(\x04\x12\x15\n\ravg_num_bytes\x18\x03 \x01(\x02\x12\x15\n\rmin_num_bytes\x18\x04 \x01(\x02\x12\x15\n\rmax_num_bytes\x18\x05 \x01(\x02\"R\n\x10StructStatistics\x12>\n\x0c\x63ommon_stats\x18\x01 \x01(\x0b\x32(.tensorflow.metadata.v0.CommonStatistics\"\xfc\x02\n\x10\x43ommonStatistics\x12\x17\n\x0fnum_non_missing\x18\x01 \x01(\x04\x12\x13\n\x0bnum_missing\x18\x02 \x01(\x04\x12\x16\n\x0emin_num_values\x18\x03 \x01(\x04\x12\x16\n\x0emax_num_values\x18\x04 \x01(\x04\x12\x16\n\x0e\x61vg_num_values\x18\x05 \x01(\x02\x12\x16\n\x0etot_num_values\x18\x08 \x01(\x04\x12?\n\x14num_values_histogram\x18\x06 \x01(\x0b\x32!.tensorflow.metadata.v0.Histogram\x12O\n\x15weighted_common_stats\x18\x07 \x01(\x0b\x32\x30.tensorflow.metadata.v0.WeightedCommonStatistics\x12H\n\x1d\x66\x65\x61ture_list_length_histogram\x18\t \x01(\x0b\x32!.tensorflow.metadata.v0.Histogram\"\xb6\x02\n\tHistogram\x12\x0f\n\x07num_nan\x18\x01 \x01(\x04\x12\x15\n\rnum_undefined\x18\x02 \x01(\x04\x12\x39\n\x07\x62uckets\x18\x03 \x03(\x0b\x32(.tensorflow.metadata.v0.Histogram.Bucket\x12=\n\x04type\x18\x04 \x01(\x0e\x32/.tensorflow.metadata.v0.Histogram.HistogramType\x12\x0c\n\x04name\x18\x05 \x01(\t\x1aK\n\x06\x42ucket\x12\x11\n\tlow_value\x18\x01 \x01(\x01\x12\x12\n\nhigh_value\x18\x02 \x01(\x01\x12\x14\n\x0csample_count\x18\x04 \x01(\x01J\x04\x08\x03\x10\x04\",\n\rHistogramType\x12\x0c\n\x08STANDARD\x10\x00\x12\r\n\tQUANTILES\x10\x01\"\xb6\x01\n\rRankHistogram\x12=\n\x07\x62uckets\x18\x01 \x03(\x0b\x32,.tensorflow.metadata.v0.RankHistogram.Bucket\x12\x0c\n\x04name\x18\x02 \x01(\t\x1aX\n\x06\x42ucket\x12\x10\n\x08low_rank\x18\x01 \x01(\x04\x12\x11\n\thigh_rank\x18\x02 \x01(\x04\x12\r\n\x05label\x18\x04 \x01(\t\x12\x14\n\x0csample_count\x18\x05 \x01(\x01J\x04\x08\x03\x10\x04\x42h\n\x1aorg.tensorflow.metadata.v0P\x01ZEgithub.com/feast-dev/feast/sdk/go/protos/tensorflow_metadata/proto/v0\xf8\x01\x01\x62\x06proto3'
  ,
  dependencies=[tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2.DESCRIPTOR,])



_FEATURENAMESTATISTICS_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='tensorflow.metadata.v0.FeatureNameStatistics.Type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FLOAT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRING', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BYTES', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRUCT', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2056,
  serialized_end=2117,
)
_sym_db.RegisterEnumDescriptor(_FEATURENAMESTATISTICS_TYPE)

_HISTOGRAM_HISTOGRAMTYPE = _descriptor.EnumDescriptor(
  name='HistogramType',
  full_name='tensorflow.metadata.v0.Histogram.HistogramType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STANDARD', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='QUANTILES', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4393,
  serialized_end=4437,
)
_sym_db.RegisterEnumDescriptor(_HISTOGRAM_HISTOGRAMTYPE)


_DATASETFEATURESTATISTICSLIST = _descriptor.Descriptor(
  name='DatasetFeatureStatisticsList',
  full_name='tensorflow.metadata.v0.DatasetFeatureStatisticsList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datasets', full_name='tensorflow.metadata.v0.DatasetFeatureStatisticsList.datasets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=114,
  serialized_end=212,
)


_DATASETFEATURESTATISTICS = _descriptor.Descriptor(
  name='DatasetFeatureStatistics',
  full_name='tensorflow.metadata.v0.DatasetFeatureStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_examples', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.num_examples', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_num_examples', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.weighted_num_examples', index=2,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.features', index=3,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cross_features', full_name='tensorflow.metadata.v0.DatasetFeatureStatistics.cross_features', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=445,
)


_CROSSFEATURESTATISTICS = _descriptor.Descriptor(
  name='CrossFeatureStatistics',
  full_name='tensorflow.metadata.v0.CrossFeatureStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path_x', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.path_x', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path_y', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.path_y', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='count', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.count', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_cross_stats', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.num_cross_stats', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='categorical_cross_stats', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.categorical_cross_stats', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='cross_stats', full_name='tensorflow.metadata.v0.CrossFeatureStatistics.cross_stats',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=448,
  serialized_end=756,
)


_NUMERICCROSSSTATISTICS = _descriptor.Descriptor(
  name='NumericCrossStatistics',
  full_name='tensorflow.metadata.v0.NumericCrossStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='correlation', full_name='tensorflow.metadata.v0.NumericCrossStatistics.correlation', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='covariance', full_name='tensorflow.metadata.v0.NumericCrossStatistics.covariance', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=823,
)


_CATEGORICALCROSSSTATISTICS = _descriptor.Descriptor(
  name='CategoricalCrossStatistics',
  full_name='tensorflow.metadata.v0.CategoricalCrossStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lift', full_name='tensorflow.metadata.v0.CategoricalCrossStatistics.lift', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=825,
  serialized_end=907,
)


_LIFTSTATISTICS = _descriptor.Descriptor(
  name='LiftStatistics',
  full_name='tensorflow.metadata.v0.LiftStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lift_series', full_name='tensorflow.metadata.v0.LiftStatistics.lift_series', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_lift_series', full_name='tensorflow.metadata.v0.LiftStatistics.weighted_lift_series', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=910,
  serialized_end=1049,
)


_LIFTSERIES_BUCKET = _descriptor.Descriptor(
  name='Bucket',
  full_name='tensorflow.metadata.v0.LiftSeries.Bucket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='low_value', full_name='tensorflow.metadata.v0.LiftSeries.Bucket.low_value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high_value', full_name='tensorflow.metadata.v0.LiftSeries.Bucket.high_value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1280,
  serialized_end=1327,
)

_LIFTSERIES_LIFTVALUE = _descriptor.Descriptor(
  name='LiftValue',
  full_name='tensorflow.metadata.v0.LiftSeries.LiftValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x_int', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_int', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x_string', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_string', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lift', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.lift', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_count', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_x_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.weighted_x_count', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x_and_y_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_and_y_count', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_x_and_y_count', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.weighted_x_and_y_count', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='x_value', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='x_count_value', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_count_value',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='x_and_y_count_value', full_name='tensorflow.metadata.v0.LiftSeries.LiftValue.x_and_y_count_value',
      index=2, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1330,
  serialized_end=1549,
)

_LIFTSERIES = _descriptor.Descriptor(
  name='LiftSeries',
  full_name='tensorflow.metadata.v0.LiftSeries',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='y_int', full_name='tensorflow.metadata.v0.LiftSeries.y_int', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y_string', full_name='tensorflow.metadata.v0.LiftSeries.y_string', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y_bucket', full_name='tensorflow.metadata.v0.LiftSeries.y_bucket', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y_count', full_name='tensorflow.metadata.v0.LiftSeries.y_count', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_y_count', full_name='tensorflow.metadata.v0.LiftSeries.weighted_y_count', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lift_values', full_name='tensorflow.metadata.v0.LiftSeries.lift_values', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_LIFTSERIES_BUCKET, _LIFTSERIES_LIFTVALUE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='y_value', full_name='tensorflow.metadata.v0.LiftSeries.y_value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='y_count_value', full_name='tensorflow.metadata.v0.LiftSeries.y_count_value',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1052,
  serialized_end=1577,
)


_FEATURENAMESTATISTICS = _descriptor.Descriptor(
  name='FeatureNameStatistics',
  full_name='tensorflow.metadata.v0.FeatureNameStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.FeatureNameStatistics.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='tensorflow.metadata.v0.FeatureNameStatistics.path', index=1,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tensorflow.metadata.v0.FeatureNameStatistics.type', index=2,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.num_stats', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.string_stats', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bytes_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.bytes_stats', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.struct_stats', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='custom_stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.custom_stats', index=7,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FEATURENAMESTATISTICS_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='field_id', full_name='tensorflow.metadata.v0.FeatureNameStatistics.field_id',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='stats', full_name='tensorflow.metadata.v0.FeatureNameStatistics.stats',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1580,
  serialized_end=2138,
)


_WEIGHTEDCOMMONSTATISTICS = _descriptor.Descriptor(
  name='WeightedCommonStatistics',
  full_name='tensorflow.metadata.v0.WeightedCommonStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_non_missing', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.num_non_missing', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_missing', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.num_missing', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_num_values', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.avg_num_values', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tot_num_values', full_name='tensorflow.metadata.v0.WeightedCommonStatistics.tot_num_values', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2140,
  serialized_end=2260,
)


_CUSTOMSTATISTIC = _descriptor.Descriptor(
  name='CustomStatistic',
  full_name='tensorflow.metadata.v0.CustomStatistic',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.CustomStatistic.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num', full_name='tensorflow.metadata.v0.CustomStatistic.num', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='str', full_name='tensorflow.metadata.v0.CustomStatistic.str', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='histogram', full_name='tensorflow.metadata.v0.CustomStatistic.histogram', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rank_histogram', full_name='tensorflow.metadata.v0.CustomStatistic.rank_histogram', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='val', full_name='tensorflow.metadata.v0.CustomStatistic.val',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2263,
  serialized_end=2452,
)


_NUMERICSTATISTICS = _descriptor.Descriptor(
  name='NumericStatistics',
  full_name='tensorflow.metadata.v0.NumericStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.NumericStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mean', full_name='tensorflow.metadata.v0.NumericStatistics.mean', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='std_dev', full_name='tensorflow.metadata.v0.NumericStatistics.std_dev', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_zeros', full_name='tensorflow.metadata.v0.NumericStatistics.num_zeros', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min', full_name='tensorflow.metadata.v0.NumericStatistics.min', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='median', full_name='tensorflow.metadata.v0.NumericStatistics.median', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max', full_name='tensorflow.metadata.v0.NumericStatistics.max', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='histograms', full_name='tensorflow.metadata.v0.NumericStatistics.histograms', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_numeric_stats', full_name='tensorflow.metadata.v0.NumericStatistics.weighted_numeric_stats', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2455,
  serialized_end=2768,
)


_STRINGSTATISTICS_FREQANDVALUE = _descriptor.Descriptor(
  name='FreqAndValue',
  full_name='tensorflow.metadata.v0.StringStatistics.FreqAndValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='tensorflow.metadata.v0.StringStatistics.FreqAndValue.value', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='frequency', full_name='tensorflow.metadata.v0.StringStatistics.FreqAndValue.frequency', index=1,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3135,
  serialized_end=3189,
)

_STRINGSTATISTICS = _descriptor.Descriptor(
  name='StringStatistics',
  full_name='tensorflow.metadata.v0.StringStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.StringStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unique', full_name='tensorflow.metadata.v0.StringStatistics.unique', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='top_values', full_name='tensorflow.metadata.v0.StringStatistics.top_values', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_length', full_name='tensorflow.metadata.v0.StringStatistics.avg_length', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rank_histogram', full_name='tensorflow.metadata.v0.StringStatistics.rank_histogram', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_string_stats', full_name='tensorflow.metadata.v0.StringStatistics.weighted_string_stats', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vocabulary_file', full_name='tensorflow.metadata.v0.StringStatistics.vocabulary_file', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STRINGSTATISTICS_FREQANDVALUE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2771,
  serialized_end=3189,
)


_WEIGHTEDNUMERICSTATISTICS = _descriptor.Descriptor(
  name='WeightedNumericStatistics',
  full_name='tensorflow.metadata.v0.WeightedNumericStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mean', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.mean', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='std_dev', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.std_dev', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='median', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.median', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='histograms', full_name='tensorflow.metadata.v0.WeightedNumericStatistics.histograms', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3192,
  serialized_end=3321,
)


_WEIGHTEDSTRINGSTATISTICS = _descriptor.Descriptor(
  name='WeightedStringStatistics',
  full_name='tensorflow.metadata.v0.WeightedStringStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='top_values', full_name='tensorflow.metadata.v0.WeightedStringStatistics.top_values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rank_histogram', full_name='tensorflow.metadata.v0.WeightedStringStatistics.rank_histogram', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3324,
  serialized_end=3488,
)


_BYTESSTATISTICS = _descriptor.Descriptor(
  name='BytesStatistics',
  full_name='tensorflow.metadata.v0.BytesStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.BytesStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unique', full_name='tensorflow.metadata.v0.BytesStatistics.unique', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_num_bytes', full_name='tensorflow.metadata.v0.BytesStatistics.avg_num_bytes', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_num_bytes', full_name='tensorflow.metadata.v0.BytesStatistics.min_num_bytes', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_num_bytes', full_name='tensorflow.metadata.v0.BytesStatistics.max_num_bytes', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3491,
  serialized_end=3657,
)


_STRUCTSTATISTICS = _descriptor.Descriptor(
  name='StructStatistics',
  full_name='tensorflow.metadata.v0.StructStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_stats', full_name='tensorflow.metadata.v0.StructStatistics.common_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3659,
  serialized_end=3741,
)


_COMMONSTATISTICS = _descriptor.Descriptor(
  name='CommonStatistics',
  full_name='tensorflow.metadata.v0.CommonStatistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_non_missing', full_name='tensorflow.metadata.v0.CommonStatistics.num_non_missing', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_missing', full_name='tensorflow.metadata.v0.CommonStatistics.num_missing', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.min_num_values', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.max_num_values', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.avg_num_values', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tot_num_values', full_name='tensorflow.metadata.v0.CommonStatistics.tot_num_values', index=5,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_values_histogram', full_name='tensorflow.metadata.v0.CommonStatistics.num_values_histogram', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weighted_common_stats', full_name='tensorflow.metadata.v0.CommonStatistics.weighted_common_stats', index=7,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_list_length_histogram', full_name='tensorflow.metadata.v0.CommonStatistics.feature_list_length_histogram', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3744,
  serialized_end=4124,
)


_HISTOGRAM_BUCKET = _descriptor.Descriptor(
  name='Bucket',
  full_name='tensorflow.metadata.v0.Histogram.Bucket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='low_value', full_name='tensorflow.metadata.v0.Histogram.Bucket.low_value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high_value', full_name='tensorflow.metadata.v0.Histogram.Bucket.high_value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_count', full_name='tensorflow.metadata.v0.Histogram.Bucket.sample_count', index=2,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4316,
  serialized_end=4391,
)

_HISTOGRAM = _descriptor.Descriptor(
  name='Histogram',
  full_name='tensorflow.metadata.v0.Histogram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_nan', full_name='tensorflow.metadata.v0.Histogram.num_nan', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_undefined', full_name='tensorflow.metadata.v0.Histogram.num_undefined', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='buckets', full_name='tensorflow.metadata.v0.Histogram.buckets', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tensorflow.metadata.v0.Histogram.type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.Histogram.name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_HISTOGRAM_BUCKET, ],
  enum_types=[
    _HISTOGRAM_HISTOGRAMTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4127,
  serialized_end=4437,
)


_RANKHISTOGRAM_BUCKET = _descriptor.Descriptor(
  name='Bucket',
  full_name='tensorflow.metadata.v0.RankHistogram.Bucket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='low_rank', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.low_rank', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high_rank', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.high_rank', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.label', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_count', full_name='tensorflow.metadata.v0.RankHistogram.Bucket.sample_count', index=3,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4534,
  serialized_end=4622,
)

_RANKHISTOGRAM = _descriptor.Descriptor(
  name='RankHistogram',
  full_name='tensorflow.metadata.v0.RankHistogram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='buckets', full_name='tensorflow.metadata.v0.RankHistogram.buckets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorflow.metadata.v0.RankHistogram.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_RANKHISTOGRAM_BUCKET, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4440,
  serialized_end=4622,
)

_DATASETFEATURESTATISTICSLIST.fields_by_name['datasets'].message_type = _DATASETFEATURESTATISTICS
_DATASETFEATURESTATISTICS.fields_by_name['features'].message_type = _FEATURENAMESTATISTICS
_DATASETFEATURESTATISTICS.fields_by_name['cross_features'].message_type = _CROSSFEATURESTATISTICS
_CROSSFEATURESTATISTICS.fields_by_name['path_x'].message_type = tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2._PATH
_CROSSFEATURESTATISTICS.fields_by_name['path_y'].message_type = tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2._PATH
_CROSSFEATURESTATISTICS.fields_by_name['num_cross_stats'].message_type = _NUMERICCROSSSTATISTICS
_CROSSFEATURESTATISTICS.fields_by_name['categorical_cross_stats'].message_type = _CATEGORICALCROSSSTATISTICS
_CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats'].fields.append(
  _CROSSFEATURESTATISTICS.fields_by_name['num_cross_stats'])
_CROSSFEATURESTATISTICS.fields_by_name['num_cross_stats'].containing_oneof = _CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats']
_CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats'].fields.append(
  _CROSSFEATURESTATISTICS.fields_by_name['categorical_cross_stats'])
_CROSSFEATURESTATISTICS.fields_by_name['categorical_cross_stats'].containing_oneof = _CROSSFEATURESTATISTICS.oneofs_by_name['cross_stats']
_CATEGORICALCROSSSTATISTICS.fields_by_name['lift'].message_type = _LIFTSTATISTICS
_LIFTSTATISTICS.fields_by_name['lift_series'].message_type = _LIFTSERIES
_LIFTSTATISTICS.fields_by_name['weighted_lift_series'].message_type = _LIFTSERIES
_LIFTSERIES_BUCKET.containing_type = _LIFTSERIES
_LIFTSERIES_LIFTVALUE.containing_type = _LIFTSERIES
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_int'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_int'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_string'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_string'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_count_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['x_and_y_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['x_and_y_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value']
_LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value'].fields.append(
  _LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_and_y_count'])
_LIFTSERIES_LIFTVALUE.fields_by_name['weighted_x_and_y_count'].containing_oneof = _LIFTSERIES_LIFTVALUE.oneofs_by_name['x_and_y_count_value']
_LIFTSERIES.fields_by_name['y_bucket'].message_type = _LIFTSERIES_BUCKET
_LIFTSERIES.fields_by_name['lift_values'].message_type = _LIFTSERIES_LIFTVALUE
_LIFTSERIES.oneofs_by_name['y_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_int'])
_LIFTSERIES.fields_by_name['y_int'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_value']
_LIFTSERIES.oneofs_by_name['y_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_string'])
_LIFTSERIES.fields_by_name['y_string'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_value']
_LIFTSERIES.oneofs_by_name['y_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_bucket'])
_LIFTSERIES.fields_by_name['y_bucket'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_value']
_LIFTSERIES.oneofs_by_name['y_count_value'].fields.append(
  _LIFTSERIES.fields_by_name['y_count'])
_LIFTSERIES.fields_by_name['y_count'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_count_value']
_LIFTSERIES.oneofs_by_name['y_count_value'].fields.append(
  _LIFTSERIES.fields_by_name['weighted_y_count'])
_LIFTSERIES.fields_by_name['weighted_y_count'].containing_oneof = _LIFTSERIES.oneofs_by_name['y_count_value']
_FEATURENAMESTATISTICS.fields_by_name['path'].message_type = tensorflow__metadata_dot_proto_dot_v0_dot_path__pb2._PATH
_FEATURENAMESTATISTICS.fields_by_name['type'].enum_type = _FEATURENAMESTATISTICS_TYPE
_FEATURENAMESTATISTICS.fields_by_name['num_stats'].message_type = _NUMERICSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['string_stats'].message_type = _STRINGSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['bytes_stats'].message_type = _BYTESSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['struct_stats'].message_type = _STRUCTSTATISTICS
_FEATURENAMESTATISTICS.fields_by_name['custom_stats'].message_type = _CUSTOMSTATISTIC
_FEATURENAMESTATISTICS_TYPE.containing_type = _FEATURENAMESTATISTICS
_FEATURENAMESTATISTICS.oneofs_by_name['field_id'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['name'])
_FEATURENAMESTATISTICS.fields_by_name['name'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['field_id']
_FEATURENAMESTATISTICS.oneofs_by_name['field_id'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['path'])
_FEATURENAMESTATISTICS.fields_by_name['path'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['field_id']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['num_stats'])
_FEATURENAMESTATISTICS.fields_by_name['num_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['string_stats'])
_FEATURENAMESTATISTICS.fields_by_name['string_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['bytes_stats'])
_FEATURENAMESTATISTICS.fields_by_name['bytes_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_FEATURENAMESTATISTICS.oneofs_by_name['stats'].fields.append(
  _FEATURENAMESTATISTICS.fields_by_name['struct_stats'])
_FEATURENAMESTATISTICS.fields_by_name['struct_stats'].containing_oneof = _FEATURENAMESTATISTICS.oneofs_by_name['stats']
_CUSTOMSTATISTIC.fields_by_name['histogram'].message_type = _HISTOGRAM
_CUSTOMSTATISTIC.fields_by_name['rank_histogram'].message_type = _RANKHISTOGRAM
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['num'])
_CUSTOMSTATISTIC.fields_by_name['num'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['str'])
_CUSTOMSTATISTIC.fields_by_name['str'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['histogram'])
_CUSTOMSTATISTIC.fields_by_name['histogram'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_CUSTOMSTATISTIC.oneofs_by_name['val'].fields.append(
  _CUSTOMSTATISTIC.fields_by_name['rank_histogram'])
_CUSTOMSTATISTIC.fields_by_name['rank_histogram'].containing_oneof = _CUSTOMSTATISTIC.oneofs_by_name['val']
_NUMERICSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_NUMERICSTATISTICS.fields_by_name['histograms'].message_type = _HISTOGRAM
_NUMERICSTATISTICS.fields_by_name['weighted_numeric_stats'].message_type = _WEIGHTEDNUMERICSTATISTICS
_STRINGSTATISTICS_FREQANDVALUE.containing_type = _STRINGSTATISTICS
_STRINGSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_STRINGSTATISTICS.fields_by_name['top_values'].message_type = _STRINGSTATISTICS_FREQANDVALUE
_STRINGSTATISTICS.fields_by_name['rank_histogram'].message_type = _RANKHISTOGRAM
_STRINGSTATISTICS.fields_by_name['weighted_string_stats'].message_type = _WEIGHTEDSTRINGSTATISTICS
_WEIGHTEDNUMERICSTATISTICS.fields_by_name['histograms'].message_type = _HISTOGRAM
_WEIGHTEDSTRINGSTATISTICS.fields_by_name['top_values'].message_type = _STRINGSTATISTICS_FREQANDVALUE
_WEIGHTEDSTRINGSTATISTICS.fields_by_name['rank_histogram'].message_type = _RANKHISTOGRAM
_BYTESSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_STRUCTSTATISTICS.fields_by_name['common_stats'].message_type = _COMMONSTATISTICS
_COMMONSTATISTICS.fields_by_name['num_values_histogram'].message_type = _HISTOGRAM
_COMMONSTATISTICS.fields_by_name['weighted_common_stats'].message_type = _WEIGHTEDCOMMONSTATISTICS
_COMMONSTATISTICS.fields_by_name['feature_list_length_histogram'].message_type = _HISTOGRAM
_HISTOGRAM_BUCKET.containing_type = _HISTOGRAM
_HISTOGRAM.fields_by_name['buckets'].message_type = _HISTOGRAM_BUCKET
_HISTOGRAM.fields_by_name['type'].enum_type = _HISTOGRAM_HISTOGRAMTYPE
_HISTOGRAM_HISTOGRAMTYPE.containing_type = _HISTOGRAM
_RANKHISTOGRAM_BUCKET.containing_type = _RANKHISTOGRAM
_RANKHISTOGRAM.fields_by_name['buckets'].message_type = _RANKHISTOGRAM_BUCKET
DESCRIPTOR.message_types_by_name['DatasetFeatureStatisticsList'] = _DATASETFEATURESTATISTICSLIST
DESCRIPTOR.message_types_by_name['DatasetFeatureStatistics'] = _DATASETFEATURESTATISTICS
DESCRIPTOR.message_types_by_name['CrossFeatureStatistics'] = _CROSSFEATURESTATISTICS
DESCRIPTOR.message_types_by_name['NumericCrossStatistics'] = _NUMERICCROSSSTATISTICS
DESCRIPTOR.message_types_by_name['CategoricalCrossStatistics'] = _CATEGORICALCROSSSTATISTICS
DESCRIPTOR.message_types_by_name['LiftStatistics'] = _LIFTSTATISTICS
DESCRIPTOR.message_types_by_name['LiftSeries'] = _LIFTSERIES
DESCRIPTOR.message_types_by_name['FeatureNameStatistics'] = _FEATURENAMESTATISTICS
DESCRIPTOR.message_types_by_name['WeightedCommonStatistics'] = _WEIGHTEDCOMMONSTATISTICS
DESCRIPTOR.message_types_by_name['CustomStatistic'] = _CUSTOMSTATISTIC
DESCRIPTOR.message_types_by_name['NumericStatistics'] = _NUMERICSTATISTICS
DESCRIPTOR.message_types_by_name['StringStatistics'] = _STRINGSTATISTICS
DESCRIPTOR.message_types_by_name['WeightedNumericStatistics'] = _WEIGHTEDNUMERICSTATISTICS
DESCRIPTOR.message_types_by_name['WeightedStringStatistics'] = _WEIGHTEDSTRINGSTATISTICS
DESCRIPTOR.message_types_by_name['BytesStatistics'] = _BYTESSTATISTICS
DESCRIPTOR.message_types_by_name['StructStatistics'] = _STRUCTSTATISTICS
DESCRIPTOR.message_types_by_name['CommonStatistics'] = _COMMONSTATISTICS
DESCRIPTOR.message_types_by_name['Histogram'] = _HISTOGRAM
DESCRIPTOR.message_types_by_name['RankHistogram'] = _RANKHISTOGRAM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatasetFeatureStatisticsList = _reflection.GeneratedProtocolMessageType('DatasetFeatureStatisticsList', (_message.Message,), {
  'DESCRIPTOR' : _DATASETFEATURESTATISTICSLIST,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.DatasetFeatureStatisticsList)
  })
_sym_db.RegisterMessage(DatasetFeatureStatisticsList)

DatasetFeatureStatistics = _reflection.GeneratedProtocolMessageType('DatasetFeatureStatistics', (_message.Message,), {
  'DESCRIPTOR' : _DATASETFEATURESTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.DatasetFeatureStatistics)
  })
_sym_db.RegisterMessage(DatasetFeatureStatistics)

CrossFeatureStatistics = _reflection.GeneratedProtocolMessageType('CrossFeatureStatistics', (_message.Message,), {
  'DESCRIPTOR' : _CROSSFEATURESTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CrossFeatureStatistics)
  })
_sym_db.RegisterMessage(CrossFeatureStatistics)

NumericCrossStatistics = _reflection.GeneratedProtocolMessageType('NumericCrossStatistics', (_message.Message,), {
  'DESCRIPTOR' : _NUMERICCROSSSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.NumericCrossStatistics)
  })
_sym_db.RegisterMessage(NumericCrossStatistics)

CategoricalCrossStatistics = _reflection.GeneratedProtocolMessageType('CategoricalCrossStatistics', (_message.Message,), {
  'DESCRIPTOR' : _CATEGORICALCROSSSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CategoricalCrossStatistics)
  })
_sym_db.RegisterMessage(CategoricalCrossStatistics)

LiftStatistics = _reflection.GeneratedProtocolMessageType('LiftStatistics', (_message.Message,), {
  'DESCRIPTOR' : _LIFTSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftStatistics)
  })
_sym_db.RegisterMessage(LiftStatistics)

LiftSeries = _reflection.GeneratedProtocolMessageType('LiftSeries', (_message.Message,), {

  'Bucket' : _reflection.GeneratedProtocolMessageType('Bucket', (_message.Message,), {
    'DESCRIPTOR' : _LIFTSERIES_BUCKET,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftSeries.Bucket)
    })
  ,

  'LiftValue' : _reflection.GeneratedProtocolMessageType('LiftValue', (_message.Message,), {
    'DESCRIPTOR' : _LIFTSERIES_LIFTVALUE,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftSeries.LiftValue)
    })
  ,
  'DESCRIPTOR' : _LIFTSERIES,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.LiftSeries)
  })
_sym_db.RegisterMessage(LiftSeries)
_sym_db.RegisterMessage(LiftSeries.Bucket)
_sym_db.RegisterMessage(LiftSeries.LiftValue)

FeatureNameStatistics = _reflection.GeneratedProtocolMessageType('FeatureNameStatistics', (_message.Message,), {
  'DESCRIPTOR' : _FEATURENAMESTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.FeatureNameStatistics)
  })
_sym_db.RegisterMessage(FeatureNameStatistics)

WeightedCommonStatistics = _reflection.GeneratedProtocolMessageType('WeightedCommonStatistics', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTEDCOMMONSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.WeightedCommonStatistics)
  })
_sym_db.RegisterMessage(WeightedCommonStatistics)

CustomStatistic = _reflection.GeneratedProtocolMessageType('CustomStatistic', (_message.Message,), {
  'DESCRIPTOR' : _CUSTOMSTATISTIC,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CustomStatistic)
  })
_sym_db.RegisterMessage(CustomStatistic)

NumericStatistics = _reflection.GeneratedProtocolMessageType('NumericStatistics', (_message.Message,), {
  'DESCRIPTOR' : _NUMERICSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.NumericStatistics)
  })
_sym_db.RegisterMessage(NumericStatistics)

StringStatistics = _reflection.GeneratedProtocolMessageType('StringStatistics', (_message.Message,), {

  'FreqAndValue' : _reflection.GeneratedProtocolMessageType('FreqAndValue', (_message.Message,), {
    'DESCRIPTOR' : _STRINGSTATISTICS_FREQANDVALUE,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.StringStatistics.FreqAndValue)
    })
  ,
  'DESCRIPTOR' : _STRINGSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.StringStatistics)
  })
_sym_db.RegisterMessage(StringStatistics)
_sym_db.RegisterMessage(StringStatistics.FreqAndValue)

WeightedNumericStatistics = _reflection.GeneratedProtocolMessageType('WeightedNumericStatistics', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTEDNUMERICSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.WeightedNumericStatistics)
  })
_sym_db.RegisterMessage(WeightedNumericStatistics)

WeightedStringStatistics = _reflection.GeneratedProtocolMessageType('WeightedStringStatistics', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTEDSTRINGSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.WeightedStringStatistics)
  })
_sym_db.RegisterMessage(WeightedStringStatistics)

BytesStatistics = _reflection.GeneratedProtocolMessageType('BytesStatistics', (_message.Message,), {
  'DESCRIPTOR' : _BYTESSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.BytesStatistics)
  })
_sym_db.RegisterMessage(BytesStatistics)

StructStatistics = _reflection.GeneratedProtocolMessageType('StructStatistics', (_message.Message,), {
  'DESCRIPTOR' : _STRUCTSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.StructStatistics)
  })
_sym_db.RegisterMessage(StructStatistics)

CommonStatistics = _reflection.GeneratedProtocolMessageType('CommonStatistics', (_message.Message,), {
  'DESCRIPTOR' : _COMMONSTATISTICS,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.CommonStatistics)
  })
_sym_db.RegisterMessage(CommonStatistics)

Histogram = _reflection.GeneratedProtocolMessageType('Histogram', (_message.Message,), {

  'Bucket' : _reflection.GeneratedProtocolMessageType('Bucket', (_message.Message,), {
    'DESCRIPTOR' : _HISTOGRAM_BUCKET,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.Histogram.Bucket)
    })
  ,
  'DESCRIPTOR' : _HISTOGRAM,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.Histogram)
  })
_sym_db.RegisterMessage(Histogram)
_sym_db.RegisterMessage(Histogram.Bucket)

RankHistogram = _reflection.GeneratedProtocolMessageType('RankHistogram', (_message.Message,), {

  'Bucket' : _reflection.GeneratedProtocolMessageType('Bucket', (_message.Message,), {
    'DESCRIPTOR' : _RANKHISTOGRAM_BUCKET,
    '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
    # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.RankHistogram.Bucket)
    })
  ,
  'DESCRIPTOR' : _RANKHISTOGRAM,
  '__module__' : 'tensorflow_metadata.proto.v0.statistics_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.metadata.v0.RankHistogram)
  })
_sym_db.RegisterMessage(RankHistogram)
_sym_db.RegisterMessage(RankHistogram.Bucket)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from datetime import datetime, timedelta

import pandas as pd
from driver_repo import driver, driver_stats_fv

from feast import FeatureStore


def main():
    pd.set_option("display.max_columns", None)
    pd.set_option("display.width", 1000)

    # Load the feature store from the current path
    fs = FeatureStore(repo_path=".")

    # Deploy the feature store to AWS
    print("Deploying feature store to AWS...")
    fs.apply([driver, driver_stats_fv])

    # Select features
    features = ["driver_hourly_stats:conv_rate", "driver_hourly_stats:acc_rate"]

    # Create an entity dataframe. This is the dataframe that will be enriched with historical features
    entity_df = pd.DataFrame(
        {
            "event_timestamp": [
                pd.Timestamp(dt, unit="ms", tz="UTC").round("ms")
                for dt in pd.date_range(
                    start=datetime.now() - timedelta(days=3),
                    end=datetime.now(),
                    periods=3,
                )
            ],
            "driver_id": [1001, 1002, 1003],
        }
    )

    print("Retrieving training data...")

    # Retrieve historical features by joining the entity dataframe to the Redshift table source
    training_df = fs.get_historical_features(
        features=features, entity_df=entity_df
    ).to_df()

    print()
    print(training_df)

    print()
    print("Loading features into the online store...")
    fs.materialize_incremental(end_date=datetime.now())

    print()
    print("Retrieving online features...")

    # Retrieve features from the online store (Firestore)
    online_features = fs.get_online_features(
        features=features, entity_rows=[{"driver_id": 1001}, {"driver_id": 1002}],
    ).to_dict()

    print()
    print(pd.DataFrame.from_dict(online_features))


if __name__ == "__main__":
    main()



# This is an example feature definition file

from google.protobuf.duration_pb2 import Duration

from feast import Entity, Feature, FeatureView, FileSource, ValueType

# Read data from parquet files. Parquet is convenient for local development mode. For
# production, you can use your favorite DWH, such as BigQuery. See Feast documentation
# for more info.
driver_hourly_stats = FileSource(
    path="%PARQUET_PATH%",
    event_timestamp_column="event_timestamp",
    created_timestamp_column="created",
)

# Define an entity for the driver. You can think of entity as a primary key used to
# fetch features.
driver = Entity(name="driver_id", value_type=ValueType.INT64, description="driver id",)

# Our parquet files contain sample data that includes a driver_id column, timestamps and
# three feature column. Here we define a Feature View that will allow us to serve this
# data to our model online.
driver_hourly_stats_view = FeatureView(
    name="driver_hourly_stats",
    entities=["driver_id"],
    ttl=Duration(seconds=86400 * 1),
    features=[
        Feature(name="conv_rate", dtype=ValueType.FLOAT),
        Feature(name="acc_rate", dtype=ValueType.FLOAT),
        Feature(name="avg_daily_trips", dtype=ValueType.INT64),
    ],
    online=True,
    batch_source=driver_hourly_stats,
    tags={},
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/OnDemandFeatureView.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.protos.feast.core import FeatureView_pb2 as feast_dot_core_dot_FeatureView__pb2
from feast.protos.feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/OnDemandFeatureView.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\030OnDemandFeatureViewProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$feast/core/OnDemandFeatureView.proto\x12\nfeast.core\x1a\x1c\x66\x65\x61st/core/FeatureView.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\"H\n\x13OnDemandFeatureView\x12\x31\n\x04spec\x18\x01 \x01(\x0b\x32#.feast.core.OnDemandFeatureViewSpec\"\xb0\x02\n\x17OnDemandFeatureViewSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12+\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32\x19.feast.core.FeatureSpecV2\x12?\n\x06inputs\x18\x04 \x03(\x0b\x32/.feast.core.OnDemandFeatureViewSpec.InputsEntry\x12>\n\x15user_defined_function\x18\x05 \x01(\x0b\x32\x1f.feast.core.UserDefinedFunction\x1aH\n\x0bInputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.feast.core.OnDemandInput:\x02\x38\x01\"\x80\x01\n\rOnDemandInput\x12/\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\x0b\x32\x17.feast.core.FeatureViewH\x00\x12\x35\n\x13request_data_source\x18\x02 \x01(\x0b\x32\x16.feast.core.DataSourceH\x00\x42\x07\n\x05input\"1\n\x13UserDefinedFunction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\x0c\x42\x61\n\x10\x66\x65\x61st.proto.coreB\x18OnDemandFeatureViewProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[feast_dot_core_dot_FeatureView__pb2.DESCRIPTOR,feast_dot_core_dot_Feature__pb2.DESCRIPTOR,feast_dot_core_dot_DataSource__pb2.DESCRIPTOR,])




_ONDEMANDFEATUREVIEW = _descriptor.Descriptor(
  name='OnDemandFeatureView',
  full_name='feast.core.OnDemandFeatureView',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spec', full_name='feast.core.OnDemandFeatureView.spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=209,
)


_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY = _descriptor.Descriptor(
  name='InputsEntry',
  full_name='feast.core.OnDemandFeatureViewSpec.InputsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.OnDemandFeatureViewSpec.InputsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.OnDemandFeatureViewSpec.InputsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=444,
  serialized_end=516,
)

_ONDEMANDFEATUREVIEWSPEC = _descriptor.Descriptor(
  name='OnDemandFeatureViewSpec',
  full_name='feast.core.OnDemandFeatureViewSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.OnDemandFeatureViewSpec.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.OnDemandFeatureViewSpec.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='feast.core.OnDemandFeatureViewSpec.features', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inputs', full_name='feast.core.OnDemandFeatureViewSpec.inputs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_defined_function', full_name='feast.core.OnDemandFeatureViewSpec.user_defined_function', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=212,
  serialized_end=516,
)


_ONDEMANDINPUT = _descriptor.Descriptor(
  name='OnDemandInput',
  full_name='feast.core.OnDemandInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_view', full_name='feast.core.OnDemandInput.feature_view', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request_data_source', full_name='feast.core.OnDemandInput.request_data_source', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='input', full_name='feast.core.OnDemandInput.input',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=519,
  serialized_end=647,
)


_USERDEFINEDFUNCTION = _descriptor.Descriptor(
  name='UserDefinedFunction',
  full_name='feast.core.UserDefinedFunction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.UserDefinedFunction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='body', full_name='feast.core.UserDefinedFunction.body', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=649,
  serialized_end=698,
)

_ONDEMANDFEATUREVIEW.fields_by_name['spec'].message_type = _ONDEMANDFEATUREVIEWSPEC
_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY.fields_by_name['value'].message_type = _ONDEMANDINPUT
_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY.containing_type = _ONDEMANDFEATUREVIEWSPEC
_ONDEMANDFEATUREVIEWSPEC.fields_by_name['features'].message_type = feast_dot_core_dot_Feature__pb2._FEATURESPECV2
_ONDEMANDFEATUREVIEWSPEC.fields_by_name['inputs'].message_type = _ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY
_ONDEMANDFEATUREVIEWSPEC.fields_by_name['user_defined_function'].message_type = _USERDEFINEDFUNCTION
_ONDEMANDINPUT.fields_by_name['feature_view'].message_type = feast_dot_core_dot_FeatureView__pb2._FEATUREVIEW
_ONDEMANDINPUT.fields_by_name['request_data_source'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE
_ONDEMANDINPUT.oneofs_by_name['input'].fields.append(
  _ONDEMANDINPUT.fields_by_name['feature_view'])
_ONDEMANDINPUT.fields_by_name['feature_view'].containing_oneof = _ONDEMANDINPUT.oneofs_by_name['input']
_ONDEMANDINPUT.oneofs_by_name['input'].fields.append(
  _ONDEMANDINPUT.fields_by_name['request_data_source'])
_ONDEMANDINPUT.fields_by_name['request_data_source'].containing_oneof = _ONDEMANDINPUT.oneofs_by_name['input']
DESCRIPTOR.message_types_by_name['OnDemandFeatureView'] = _ONDEMANDFEATUREVIEW
DESCRIPTOR.message_types_by_name['OnDemandFeatureViewSpec'] = _ONDEMANDFEATUREVIEWSPEC
DESCRIPTOR.message_types_by_name['OnDemandInput'] = _ONDEMANDINPUT
DESCRIPTOR.message_types_by_name['UserDefinedFunction'] = _USERDEFINEDFUNCTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

OnDemandFeatureView = _reflection.GeneratedProtocolMessageType('OnDemandFeatureView', (_message.Message,), {
  'DESCRIPTOR' : _ONDEMANDFEATUREVIEW,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.OnDemandFeatureView)
  })
_sym_db.RegisterMessage(OnDemandFeatureView)

OnDemandFeatureViewSpec = _reflection.GeneratedProtocolMessageType('OnDemandFeatureViewSpec', (_message.Message,), {

  'InputsEntry' : _reflection.GeneratedProtocolMessageType('InputsEntry', (_message.Message,), {
    'DESCRIPTOR' : _ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY,
    '__module__' : 'feast.core.OnDemandFeatureView_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.OnDemandFeatureViewSpec.InputsEntry)
    })
  ,
  'DESCRIPTOR' : _ONDEMANDFEATUREVIEWSPEC,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.OnDemandFeatureViewSpec)
  })
_sym_db.RegisterMessage(OnDemandFeatureViewSpec)
_sym_db.RegisterMessage(OnDemandFeatureViewSpec.InputsEntry)

OnDemandInput = _reflection.GeneratedProtocolMessageType('OnDemandInput', (_message.Message,), {
  'DESCRIPTOR' : _ONDEMANDINPUT,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.OnDemandInput)
  })
_sym_db.RegisterMessage(OnDemandInput)

UserDefinedFunction = _reflection.GeneratedProtocolMessageType('UserDefinedFunction', (_message.Message,), {
  'DESCRIPTOR' : _USERDEFINEDFUNCTION,
  '__module__' : 'feast.core.OnDemandFeatureView_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.UserDefinedFunction)
  })
_sym_db.RegisterMessage(UserDefinedFunction)


DESCRIPTOR._options = None
_ONDEMANDFEATUREVIEWSPEC_INPUTSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2021 The Feast Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from datetime import datetime
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union

from pydantic import StrictStr
from pydantic.typing import Literal

from feast import Entity, FeatureTable, FeatureView, utils
from feast.infra.online_stores.helpers import compute_entity_id
from feast.infra.online_stores.online_store import OnlineStore
from feast.protos.feast.types.EntityKey_pb2 import EntityKey as EntityKeyProto
from feast.protos.feast.types.Value_pb2 import Value as ValueProto
from feast.repo_config import FeastConfigBaseModel, RepoConfig
from feast.usage import log_exceptions_and_usage, tracing_span

try:
    import boto3
    from botocore.exceptions import ClientError
except ImportError as e:
    from feast.errors import FeastExtrasDependencyImportError

    raise FeastExtrasDependencyImportError("aws", str(e))


class DynamoDBOnlineStoreConfig(FeastConfigBaseModel):
    """Online store config for DynamoDB store"""

    type: Literal["dynamodb"] = "dynamodb"
    """Online store type selector"""

    region: StrictStr
    """ AWS Region Name """


class DynamoDBOnlineStore(OnlineStore):
    """
    Online feature store for AWS DynamoDB.
    """

    @log_exceptions_and_usage(online_store="dynamodb")
    def update(
        self,
        config: RepoConfig,
        tables_to_delete: Sequence[Union[FeatureTable, FeatureView]],
        tables_to_keep: Sequence[Union[FeatureTable, FeatureView]],
        entities_to_delete: Sequence[Entity],
        entities_to_keep: Sequence[Entity],
        partial: bool,
    ):
        online_config = config.online_store
        assert isinstance(online_config, DynamoDBOnlineStoreConfig)
        dynamodb_client, dynamodb_resource = self._initialize_dynamodb(online_config)

        for table_instance in tables_to_keep:
            try:
                dynamodb_resource.create_table(
                    TableName=f"{config.project}.{table_instance.name}",
                    KeySchema=[{"AttributeName": "entity_id", "KeyType": "HASH"}],
                    AttributeDefinitions=[
                        {"AttributeName": "entity_id", "AttributeType": "S"}
                    ],
                    BillingMode="PAY_PER_REQUEST",
                )
            except ClientError as ce:
                # If the table creation fails with ResourceInUseException,
                # it means the table already exists or is being created.
                # Otherwise, re-raise the exception
                if ce.response["Error"]["Code"] != "ResourceInUseException":
                    raise

        for table_instance in tables_to_keep:
            dynamodb_client.get_waiter("table_exists").wait(
                TableName=f"{config.project}.{table_instance.name}"
            )

        self._delete_tables_idempotent(dynamodb_resource, config, tables_to_delete)

    def teardown(
        self,
        config: RepoConfig,
        tables: Sequence[Union[FeatureTable, FeatureView]],
        entities: Sequence[Entity],
    ):
        online_config = config.online_store
        assert isinstance(online_config, DynamoDBOnlineStoreConfig)
        _, dynamodb_resource = self._initialize_dynamodb(online_config)

        self._delete_tables_idempotent(dynamodb_resource, config, tables)

    @log_exceptions_and_usage(online_store="dynamodb")
    def online_write_batch(
        self,
        config: RepoConfig,
        table: Union[FeatureTable, FeatureView],
        data: List[
            Tuple[EntityKeyProto, Dict[str, ValueProto], datetime, Optional[datetime]]
        ],
        progress: Optional[Callable[[int], Any]],
    ) -> None:
        online_config = config.online_store
        assert isinstance(online_config, DynamoDBOnlineStoreConfig)
        _, dynamodb_resource = self._initialize_dynamodb(online_config)

        table_instance = dynamodb_resource.Table(f"{config.project}.{table.name}")
        with table_instance.batch_writer() as batch:
            for entity_key, features, timestamp, created_ts in data:
                entity_id = compute_entity_id(entity_key)
                batch.put_item(
                    Item={
                        "entity_id": entity_id,  # PartitionKey
                        "event_ts": str(utils.make_tzaware(timestamp)),
                        "values": {
                            k: v.SerializeToString()
                            for k, v in features.items()  # Serialized Features
                        },
                    }
                )
                if progress:
                    progress(1)

    @log_exceptions_and_usage(online_store="dynamodb")
    def online_read(
        self,
        config: RepoConfig,
        table: Union[FeatureTable, FeatureView],
        entity_keys: List[EntityKeyProto],
        requested_features: Optional[List[str]] = None,
    ) -> List[Tuple[Optional[datetime], Optional[Dict[str, ValueProto]]]]:
        online_config = config.online_store
        assert isinstance(online_config, DynamoDBOnlineStoreConfig)
        _, dynamodb_resource = self._initialize_dynamodb(online_config)

        result: List[Tuple[Optional[datetime], Optional[Dict[str, ValueProto]]]] = []
        for entity_key in entity_keys:
            table_instance = dynamodb_resource.Table(f"{config.project}.{table.name}")
            entity_id = compute_entity_id(entity_key)
            with tracing_span(name="remote_call"):
                response = table_instance.get_item(Key={"entity_id": entity_id})
            value = response.get("Item")

            if value is not None:
                res = {}
                for feature_name, value_bin in value["values"].items():
                    val = ValueProto()
                    val.ParseFromString(value_bin.value)
                    res[feature_name] = val
                result.append((value["event_ts"], res))
            else:
                result.append((None, None))
        return result

    def _initialize_dynamodb(self, online_config: DynamoDBOnlineStoreConfig):
        return (
            boto3.client("dynamodb", region_name=online_config.region),
            boto3.resource("dynamodb", region_name=online_config.region),
        )

    def _delete_tables_idempotent(
        self,
        dynamodb_resource,
        config: RepoConfig,
        tables: Sequence[Union[FeatureTable, FeatureView]],
    ):
        for table_instance in tables:
            try:
                table = dynamodb_resource.Table(
                    f"{config.project}.{table_instance.name}"
                )
                table.delete()
            except ClientError as ce:
                # If the table deletion fails with ResourceNotFoundException,
                # it means the table has already been deleted.
                # Otherwise, re-raise the exception
                if ce.response["Error"]["Code"] != "ResourceNotFoundException":
                    raise



import os
import tempfile
from datetime import datetime, timedelta
from pathlib import Path

import pandas as pd
from pytz import utc

import feast.driver_test_data as driver_data
from feast import FeatureStore
from tests.utils.cli_utils import CliRunner, get_example_repo


def _get_last_feature_row(df: pd.DataFrame, driver_id, max_date: datetime):
    """ Manually extract last feature value from a dataframe for a given driver_id with up to `max_date` date """
    filtered = df[
        (df["driver_id"] == driver_id)
        & (df["event_timestamp"] < max_date.replace(tzinfo=utc))
    ]
    max_ts = filtered.loc[filtered["event_timestamp"].idxmax()]["event_timestamp"]
    filtered_by_ts = filtered[filtered["event_timestamp"] == max_ts]
    return filtered_by_ts.loc[filtered_by_ts["created"].idxmax()]


def _assert_online_features(
    store: FeatureStore, driver_df: pd.DataFrame, max_date: datetime
):
    """Assert that features in online store are up to date with `max_date` date."""
    # Read features back
    response = store.get_online_features(
        features=[
            "driver_hourly_stats:conv_rate",
            "driver_hourly_stats:avg_daily_trips",
            "global_daily_stats:num_rides",
            "global_daily_stats:avg_ride_length",
        ],
        entity_rows=[{"driver_id": 1001}],
        full_feature_names=True,
    )

    # Float features should still be floats from the online store...
    assert (
        response.field_values[0].fields["driver_hourly_stats__conv_rate"].float_val > 0
    )

    result = response.to_dict()
    assert len(result) == 5
    assert "driver_hourly_stats__avg_daily_trips" in result
    assert "driver_hourly_stats__conv_rate" in result
    assert (
        abs(
            result["driver_hourly_stats__conv_rate"][0]
            - _get_last_feature_row(driver_df, 1001, max_date)["conv_rate"]
        )
        < 0.01
    )
    assert "global_daily_stats__num_rides" in result
    assert "global_daily_stats__avg_ride_length" in result


def test_e2e_local() -> None:
    """
    A more comprehensive than "basic" test, using local provider.

    1. Create a repo.
    2. Apply
    3. Ingest some data to online store from parquet
    4. Read from the online store to make sure it made it there.
    """

    runner = CliRunner()
    with tempfile.TemporaryDirectory() as data_dir:

        # Generate some test data in parquet format.
        end_date = datetime.now().replace(microsecond=0, second=0, minute=0)
        start_date = end_date - timedelta(days=15)

        driver_entities = [1001, 1002, 1003, 1004, 1005]
        driver_df = driver_data.create_driver_hourly_stats_df(
            driver_entities, start_date, end_date
        )
        driver_stats_path = os.path.join(data_dir, "driver_stats.parquet")
        driver_df.to_parquet(path=driver_stats_path, allow_truncated_timestamps=True)

        global_df = driver_data.create_global_daily_stats_df(start_date, end_date)
        global_stats_path = os.path.join(data_dir, "global_stats.parquet")
        global_df.to_parquet(path=global_stats_path, allow_truncated_timestamps=True)

        # Note that runner takes care of running apply/teardown for us here.
        # We patch python code in example_feature_repo_2.py to set the path to Parquet files.
        with runner.local_repo(
            get_example_repo("example_feature_repo_2.py")
            .replace("%PARQUET_PATH%", driver_stats_path)
            .replace("%PARQUET_PATH_GLOBAL%", global_stats_path),
            "file",
        ) as store:

            assert store.repo_path is not None

            # feast materialize
            r = runner.run(
                [
                    "materialize",
                    start_date.isoformat(),
                    (end_date - timedelta(days=7)).isoformat(),
                ],
                cwd=Path(store.repo_path),
            )

            assert r.returncode == 0

            _assert_online_features(store, driver_df, end_date - timedelta(days=7))

            # feast materialize-incremental
            r = runner.run(
                ["materialize-incremental", end_date.isoformat()],
                cwd=Path(store.repo_path),
            )

            assert r.returncode == 0

            _assert_online_features(store, driver_df, end_date)

        # Test a failure case when the parquet file doesn't include a join key
        with runner.local_repo(
            get_example_repo("example_feature_repo_with_entity_join_key.py").replace(
                "%PARQUET_PATH%", driver_stats_path
            ),
            "file",
        ) as store:

            assert store.repo_path is not None

            # feast materialize
            returncode, output = runner.run_with_output(
                [
                    "materialize",
                    start_date.isoformat(),
                    (end_date - timedelta(days=7)).isoformat(),
                ],
                cwd=Path(store.repo_path),
            )

            assert returncode != 0
            assert "feast.errors.FeastJoinKeysDuringMaterialization" in str(output)



from contextlib import contextmanager
from pathlib import Path
from tempfile import TemporaryDirectory
from textwrap import dedent
from typing import Optional

import assertpy

from feast.repo_operations import get_ignore_files, get_repo_files, read_feastignore


@contextmanager
def feature_repo(feastignore_contents: Optional[str]):
    with TemporaryDirectory() as tmp_dir:
        repo_root = Path(tmp_dir)
        (repo_root / "foo").mkdir()
        (repo_root / "foo1").mkdir()
        (repo_root / "foo1/bar").mkdir()
        (repo_root / "bar").mkdir()
        (repo_root / "bar/subdir1").mkdir()
        (repo_root / "bar/subdir1/subdir2").mkdir()

        (repo_root / "a.py").touch()
        (repo_root / "foo/b.py").touch()
        (repo_root / "foo1/c.py").touch()
        (repo_root / "foo1/bar/d.py").touch()
        (repo_root / "bar/e.py").touch()
        (repo_root / "bar/subdir1/f.py").touch()
        (repo_root / "bar/subdir1/subdir2/g.py").touch()

        if feastignore_contents:
            with open(repo_root / ".feastignore", "w") as f:
                f.write(feastignore_contents)

        yield repo_root


def test_feastignore_no_file():
    # Tests feature repo without .feastignore file
    with feature_repo(None) as repo_root:
        assertpy.assert_that(read_feastignore(repo_root)).is_equal_to([])
        assertpy.assert_that(get_ignore_files(repo_root, [])).is_equal_to(set())
        assertpy.assert_that(get_repo_files(repo_root)).is_equal_to(
            [
                (repo_root / "a.py").resolve(),
                (repo_root / "bar/e.py").resolve(),
                (repo_root / "bar/subdir1/f.py").resolve(),
                (repo_root / "bar/subdir1/subdir2/g.py").resolve(),
                (repo_root / "foo/b.py").resolve(),
                (repo_root / "foo1/bar/d.py").resolve(),
                (repo_root / "foo1/c.py").resolve(),
            ]
        )


def test_feastignore_no_stars():
    # Tests .feastignore that doesn't contain "*" in paths
    feastignore_contents = dedent(
        """
        # We can put some comments here

        foo # match directory
        bar/subdir1/f.py # match specific file
    """
    )
    with feature_repo(feastignore_contents) as repo_root:
        ignore_paths = ["foo", "bar/subdir1/f.py"]
        assertpy.assert_that(read_feastignore(repo_root)).is_equal_to(ignore_paths)
        assertpy.assert_that(get_ignore_files(repo_root, ignore_paths)).is_equal_to(
            {
                (repo_root / "foo/b.py").resolve(),
                (repo_root / "bar/subdir1/f.py").resolve(),
            }
        )
        assertpy.assert_that(get_repo_files(repo_root)).is_equal_to(
            [
                (repo_root / "a.py").resolve(),
                (repo_root / "bar/e.py").resolve(),
                (repo_root / "bar/subdir1/subdir2/g.py").resolve(),
                (repo_root / "foo1/bar/d.py").resolve(),
                (repo_root / "foo1/c.py").resolve(),
            ]
        )


def test_feastignore_with_stars():
    # Tests .feastignore that contains "*" and "**" in paths
    feastignore_contents = dedent(
        """
        foo/*.py # match python files directly under foo/
        bar/**   # match everything (recursively) under bar/
        */c.py   # match c.py in any directory
        */d.py   # match d.py in any directory (this shouldn't match anything)
    """
    )
    with feature_repo(feastignore_contents) as repo_root:
        ignore_paths = ["foo/*.py", "bar/**", "*/c.py", "*/d.py"]
        assertpy.assert_that(read_feastignore(repo_root)).is_equal_to(ignore_paths)
        assertpy.assert_that(get_ignore_files(repo_root, ignore_paths)).is_equal_to(
            {
                (repo_root / "foo/b.py").resolve(),
                (repo_root / "bar/subdir1/f.py").resolve(),
                (repo_root / "bar/e.py").resolve(),
                (repo_root / "bar/subdir1/f.py").resolve(),
                (repo_root / "bar/subdir1/subdir2/g.py").resolve(),
                (repo_root / "foo1/c.py").resolve(),
            }
        )
        assertpy.assert_that(get_repo_files(repo_root)).is_equal_to(
            [(repo_root / "a.py").resolve(), (repo_root / "foo1/bar/d.py").resolve()]
        )


def test_feastignore_with_stars2():
    # Another test of .feastignore that contains "**" in paths
    feastignore_contents = dedent(
        """
        # match everything (recursively) that has "bar" in its path
        **/bar/**
    """
    )
    with feature_repo(feastignore_contents) as repo_root:
        ignore_paths = ["**/bar/**"]
        assertpy.assert_that(read_feastignore(repo_root)).is_equal_to(ignore_paths)
        assertpy.assert_that(get_ignore_files(repo_root, ignore_paths)).is_equal_to(
            {
                (repo_root / "bar/subdir1/f.py").resolve(),
                (repo_root / "bar/e.py").resolve(),
                (repo_root / "bar/subdir1/f.py").resolve(),
                (repo_root / "bar/subdir1/subdir2/g.py").resolve(),
                (repo_root / "foo1/bar/d.py").resolve(),
            }
        )
        assertpy.assert_that(get_repo_files(repo_root)).is_equal_to(
            [
                (repo_root / "a.py").resolve(),
                (repo_root / "foo/b.py").resolve(),
                (repo_root / "foo1/c.py").resolve(),
            ]
        )



import pytest

from feast.errors import FeatureNameCollisionError
from feast.feature_store import _validate_feature_refs


def test_feature_name_collision_on_historical_retrieval():
    # _validate_feature_refs is the function that checks for colliding feature names
    # check when feature names collide and 'full_feature_names=False'
    with pytest.raises(FeatureNameCollisionError) as error:
        _validate_feature_refs(
            feature_refs=[
                "driver_stats:conv_rate",
                "driver_stats:avg_daily_trips",
                "customer_profile:current_balance",
                "customer_profile:avg_passenger_count",
                "customer_profile:lifetime_trip_count",
                "customer_profile:avg_daily_trips",
            ],
            full_feature_names=False,
        )

        expected_error_message = (
            "Duplicate features named avg_daily_trips found.\n"
            "To resolve this collision, either use the full feature name by setting "
            "'full_feature_names=True', or ensure that the features in question have different names."
        )

        assert str(error.value) == expected_error_message

    # check when feature names collide and 'full_feature_names=True'
    with pytest.raises(FeatureNameCollisionError) as error:
        _validate_feature_refs(
            feature_refs=[
                "driver_stats:conv_rate",
                "driver_stats:avg_daily_trips",
                "driver_stats:avg_daily_trips",
                "customer_profile:current_balance",
                "customer_profile:avg_passenger_count",
                "customer_profile:lifetime_trip_count",
                "customer_profile:avg_daily_trips",
            ],
            full_feature_names=True,
        )

        expected_error_message = (
            "Duplicate features named driver_stats__avg_daily_trips found.\n"
            "To resolve this collision, please ensure that the feature views or their own features "
            "have different names. If you're intentionally joining the same feature view twice on "
            "different sets of entities, please rename one of the feature views with '.with_name'."
        )
        assert str(error.value) == expected_error_message
